// Find the best hand a player can have. Does this by checking every possible hand.
// Inefficient, but works fairly well.
// PARAMETERS: hand - the hand the player has. Not the common cards
//             stringRet - a boolean to determine if they want just the hand type or the result
function checkPossibleHands (hand, stringRet) {
	var cards = [hand[0], hand[1], game.river[0], game.river[1], 
		game.river[2], game.river[3], game.river[4]];
	
	cards.sort(
		function(a, b) {
			var aVal;
			var bVal;
			if (a.val == 'T')
				a.val = 10;
			else if (a.val == 'J')
				a.val = 11;
			else if (a.val == 'Q')
				a.val = 12;
			else if (a.val == 'K')
				a.val = 13;
			else if (a.val == 'A')
				a.val = 1;
			aVal = a.val;
				
			if (b.val == 'T')
				b.val = 10;
			else if (b.val == 'J')
				b.val = 11;
			else if (b.val == 'Q')
				b.val = 12;
			else if (b.val == 'K')
				b.val = 13;
			else if (b.val == 'A')
				b.val = 1;
			bVal = b.val;
				
			return aVal - bVal;
		}
	);
	
	var bestHand = {
		handType: 0,
		handHigh: 0
	};
	var hand2Check = {
		handType: 0,
		handHigh: 0
	};
	
	var counter = 0;
	for ( var i = 0; i < cards.length-1 ; i++ )
	{
		for ( var j = i+1; j < cards.length ; j++ )
		{
			var cardClone = cards.slice(0);
			cardClone.splice(j,1);
			cardClone.splice(i,1);
			hand2Check = checkHand(cardClone);
			bestHand = compare2Hands(bestHand, hand2Check);
			counter++;
		}
	}
	
	if (stringRet)
	{
		return ( handTypeToString(bestHand.handType) ); 
	}
	else
	{
		return bestHand;
	}
}

// Find the best hand a player can have when there is no river card yet. Does this by checking every possible hand.
// Inefficient, but works fairly well.
function checkPossibleHandsNoRiver (hand, stringRet) {
	var cards = [hand[0], hand[1], game.river[0], game.river[1], 
		game.river[2], game.river[3]];
	
	cards.sort(
		function(a, b) {
			var aVal;
			var bVal;
			if (a.val == 'T')
				a.val = 10;
			else if (a.val == 'J')
				a.val = 11;
			else if (a.val == 'Q')
				a.val = 12;
			else if (a.val == 'K')
				a.val = 13;
			else if (a.val == 'A')
				a.val = 1;
			aVal = a.val;
				
			if (b.val == 'T')
				b.val = 10;
			else if (b.val == 'J')
				b.val = 11;
			else if (b.val == 'Q')
				b.val = 12;
			else if (b.val == 'K')
				b.val = 13;
			else if (b.val == 'A')
				b.val = 1;
			bVal = b.val;
				
			return aVal - bVal;
		}
	);
	
	var bestHand = {
		handType: 0,
		handHigh: 0
	};
	var hand2Check = {
		handType: 0,
		handHigh: 0
	};
	
	for ( var i = 0; i < cards.length ; i++ )
	{
		var cardClone = cards.slice(0);
		cardClone.splice(i,1);
		hand2Check = checkHand(cardClone);
		bestHand = compare2Hands(bestHand, hand2Check);		
	}	
	
	if (stringRet)
	{
		return ( handTypeToString(bestHand.handType) ); 
	}
	else
	{
		return bestHand;
	}
}

// Find the type of hand a player has after the flop.
function checkPossibleHandsFlopOnly (hand, stringRet) {
	var cards = [hand[0], hand[1], game.river[0], game.river[1], game.river[2]];
		
	cards.sort(
		function(a, b) {
			var aVal;
			var bVal;
			if (a.val == 'T')
				a.val = 10;
			else if (a.val == 'J')
				a.val = 11;
			else if (a.val == 'Q')
				a.val = 12;
			else if (a.val == 'K')
				a.val = 13;
			else if (a.val == 'A')
				a.val = 1;
			aVal = a.val;
				
			if (b.val == 'T')
				b.val = 10;
			else if (b.val == 'J')
				b.val = 11;
			else if (b.val == 'Q')
				b.val = 12;
			else if (b.val == 'K')
				b.val = 13;
			else if (b.val == 'A')
				b.val = 1;
			bVal = b.val;
				
			return aVal - bVal;
		}
	);
		
	var bestHand = {
		handType: 0,
		handHigh: 0
	};
	
	var cardClone = cards.slice(0);
	bestHand = checkHand(cardClone);
	
	if (stringRet)
	{
		return ( handTypeToString(bestHand.handType) ); 
	}
	else
	{
		return bestHand;
	}
}

// Just grabs the return object for the hand type and returns a string for what type of hand it is.
// Simple switch statement. 
function handTypeToString (handType) {
	switch (handType)
	{
		case 9:
			return "Royal Flush";
		case 8:
			return "Straight Flush";
		case 7:
			return "Four of a kind";
		case 6: 
			return "Full House";
		case 5: 
			return "Flush";
		case 4: 
			return "Straight";
		case 3: 
			return "Three of a kind";
		case 2: 
			return "Two Pair";
		case 1: 
			return "Pair";
		case 0: 
			return "High Card";
	}
}

// Find the type of hand a set of 5 cards has. 
function checkHand(cards) {
	var suits = [checkMyType(cards, 's'), checkMyType(cards, 'c'), checkMyType(cards, 'h'), checkMyType(cards, 'd')];
	
 	if ( hasRoyalFlush ( cards, suits ) != 0 )
	{
		return hasRoyalFlush ( cards, suits );
	}
	else if ( hasStraightFlush(cards, suits) != 0 )
	{
		return hasStraightFlush(cards, suits);
	}
	else if ( hasFourOfAKind(cards) != 0 )
	{
		return hasFourOfAKind(cards);
	}
	else if ( hasFullHouse(cards) != 0 )
	{
		return hasFullHouse(cards);
	}
	else if ( hasFlush(cards, suits) != 0 )
	{
		return hasFlush(cards, suits);
	}
	else if ( hasStraight(cards) != 0 )
	{
		return hasStraight(cards);
	}
	else if ( hasThreeOfAKind(cards) != 0 )
	{
		return hasThreeOfAKind(cards);
	}
	else if ( hasTwoPair(cards) != 0 )
	{
		return hasTwoPair(cards);
	}
	else if ( hasPair(cards) != 0 )
	{
		return hasPair(cards);
	}
	else // high card
	{
		return hasHighCard(cards);
	}
	
	alert ("Error. Returning royal flush");
	return {
		handType:9
	}
}

// Determines which hand is better by looking through the handResults.handType's.
// Depending on tie cases and what the handType is, has some extra checkers.
// TODO: Expand this to check more tie-breaking cases, but only after the group 
//         of "has" functions return more thorough objects.
function compare2Hands (firstHandResult, secondHandResult)
{
	if ( firstHandResult.handType > secondHandResult.handType )
	{
		return firstHandResult;
	}
	else if ( secondHandResult.handType > firstHandResult.handType )
	{
		return secondHandResult;
	}
	else // hands are the same type, need to compare
	{
		// 9. Royal Flush. Can't tie
		// 8. Straight Flush. Judge by highCard
		// 7. Four of a kind. Judge by matching, then by kicker
		// 6. Full House. Judge by threeMatch, then by twoMatch
		// 5. Flush. Judge by highest card. Need to improve by adding more
		// 4. Straight. Judge by highCard
		// 3. Three of a kind. Judge by threeOfAKindVal. Need to improve by adding highest cards
		// 2. Two Pair. Judge by highPair, then by lowPair
		// 1. Pair. Judge by pairVal.
		// 0. High Card. Judge by handHigh
		if ( firstHandResult.handType == 8 )
		{
			if ( firstHandResult.highCard > secondHandResult.highCard )
			{
				return firstHandResult;
			}
			else if ( secondHandResult.highCard > firstHandResult.highCard )
			{
				return secondHandResult;
			}
			else
			{
				return firstHandResult; // Draw scenario. Just return the first case since they're even.
			}
		}
		else if ( firstHandResult.handType == 7 )
		{
			if ( firstHandResult.matching > secondHandResult.matching )
			{
				return firstHandResult;
			}
			else if ( firstHandResult.matching < secondHandResult.matching )
			{
				return secondHandResult;
			}
			else if ( firstHandResult.kicker > secondHandResult.kicker )
			{
				return firstHandResult;
			}
			else if ( firstHandResult.kicker < secondHandResult.kicker )
			{
				return secondHandResult;
			}
			else
			{
				return firstHandResult; // Even case. Just choose the first hand.
			}
		}
		else if ( firstHandResult.handType == 6 )
		{
			if ( firstHandResult.threeMatch > secondHandResult.threeMatch ) 
			{
				return firstHandResult;
			}
			else if ( secondHandResult.threeMatch > firstHandResult.threeMatch )
			{
				return secondHandResult;
			}
			else if ( firstHandResult.twoMatch > secondHandResult.twoMatch ) 
			{
				return firstHandResult;
			}
			else if ( secondHandResult.twoMatch > firstHandResult.twoMatch )
			{
				return secondHandResult;
			}
			else
			{
				return firstHandResult; // Arbitrary. Both hands are equal
			}
			
		}
		else if ( firstHandResult.handType == 5 )
		{
			if ( firstHandResult.highCard > secondHandResult.highCard )
			{
				return firstHandResult;
			}
			else if ( firstHandResult.highCard < secondHandResult.highCard )
			{
				return secondHandResult;
			}
			else
			{
				return firstHandResult;
			}
		}
		else if ( firstHandResult.handType == 4 )
		{
			if ( firstHandResult.highCard > secondHandResult.highCard )
			{
				return firstHandResult;
			}
			else if ( firstHandResult.highCard < secondHandResult.highCard )
			{
				return secondHandResult;
			}
			else
			{
				return firstHandResult;
			}
		}
		else if ( firstHandResult.handType == 3 )
		{
			if ( firstHandResult.threeOfAKindVal > secondHandResult.threeOfAKindVal )
			{
				return firstHandResult;
			}
			else if ( firstHandResult.threeOfAKindVal < secondHandResult.threeOfAKindVal )
			{
				return secondHandResult;
			}
			else
			{
				return firstHandResult;
			}
		}
		else if ( firstHandResult.handType == 2 )
		{
			if ( firstHandResult.highPair > secondHandResult.highPair )
			{
				return firstHandResult;
			}
			else if ( firstHandResult.highPair < secondHandResult.highPair )
			{
				return secondHandResult;
			}
			else if ( firstHandResult.lowPair > secondHandResult.lowPair )
			{
				return firstHandResult;
			}
			else if ( firstHandResult.lowPair < secondHandResult.lowPair )
			{
				return secondHandResult;
			}
			else
			{
				return firstHandResult;
			}
		}
		else if ( firstHandResult.handType == 1 )
		{
			if ( firstHandResult.pairVal > secondHandResult.pairVal )
			{
				return firstHandResult;
			}
			else if ( firstHandResult.pairVal < secondHandResult.pairVal )
			{
				return secondHandResult;
			}
			else
			{
				return firstHandResult;
			}
		}
		else if ( firstHandResult.handType == 0 )
		{
			if ( firstHandResult.handHigh > secondHandResult.handHigh )
			{
				return firstHandResult;
			}
			else if ( firstHandResult.handHigh < secondHandResult.handHigh )
			{
				return secondHandResult;
			}
			else
			{
				return firstHandResult;
			}
		}
	}

	return firstHandResult; // sort of a catch in case something breaks
}

// Determines which player has the better hand. 
// Returns: -1 if first hand given, 0 if tie, and 1 if second hand given
function whoHasBetterHand (firstHandResult, secondHandResult)
{
		if ( firstHandResult.handType > secondHandResult.handType )
	{
		return -1;
	}
	else if ( secondHandResult.handType > firstHandResult.handType )
	{
		return 1;
	}
	else // hands are the same type, need to compare the details
	{
		// 9. Royal Flush. Can't tie
		// 8. Straight Flush. Judge by highCard
		// 7. Four of a kind. Judge by matching, then by kicker
		// 6. Full House. Judge by threeMatch, then by twoMatch
		// 5. Flush. Judge by highest card. Need to improve by adding more
		// 4. Straight. Judge by highCard
		// 3. Three of a kind. Judge by threeOfAKindVal. Need to improve by adding highest cards
		// 2. Two Pair. Judge by highPair, then by lowPair
		// 1. Pair. Judge by pairVal.
		// 0. High Card. Judge by handHigh
		if ( firstHandResult.handType == 8 )
		{
			if ( firstHandResult.highCard > secondHandResult.highCard )
			{
				return -1;
			}
			else if ( secondHandResult.highCard > firstHandResult.highCard )
			{
				return 1;
			}
			else
			{
				return 0; // Draw scenario. Just return the first case since they're even.
			}
		}
		else if ( firstHandResult.handType == 7 )
		{
			if ( firstHandResult.matching > secondHandResult.matching )
			{
				return -1;
			}
			else if ( firstHandResult.matching < secondHandResult.matching )
			{
				return 1;
			}
			else if ( firstHandResult.kicker > secondHandResult.kicker )
			{
				return -1;
			}
			else if ( firstHandResult.kicker < secondHandResult.kicker )
			{
				return 1;
			}
			else
			{
				return 0;
			}
		}
		else if ( firstHandResult.handType == 6 )
		{
			if ( firstHandResult.threeMatch > secondHandResult.threeMatch ) 
			{
				return -1;
			}
			else if ( secondHandResult.threeMatch > firstHandResult.threeMatch )
			{
				return 1;
			}
			else if ( firstHandResult.twoMatch > secondHandResult.twoMatch ) 
			{
				return -1;
			}
			else if ( secondHandResult.twoMatch > firstHandResult.twoMatch )
			{
				return 1;
			}
			else
			{
				return 0; // Arbitrary. Both hands are equal
			}
			
		}
		else if ( firstHandResult.handType == 5 )
		{
			if ( firstHandResult.highCard > secondHandResult.highCard )
			{
				return -1;
			}
			else if ( firstHandResult.highCard < secondHandResult.highCard )
			{
				return 1;
			}
			else
			{
				return 0;
			}
		}
		else if ( firstHandResult.handType == 4 )
		{
			if ( firstHandResult.highCard > secondHandResult.highCard )
			{
				return -1;
			}
			else if ( firstHandResult.highCard < secondHandResult.highCard )
			{
				return 1;
			}
			else
			{
				return 0;
			}
		}
		else if ( firstHandResult.handType == 3 )
		{
			if ( firstHandResult.threeOfAKindVal > secondHandResult.threeOfAKindVal )
			{
				return -1;
			}
			else if ( firstHandResult.threeOfAKindVal < secondHandResult.threeOfAKindVal )
			{
				return 1;
			}
			else
			{
				return 0;
			}
		}
		else if ( firstHandResult.handType == 2 )
		{
			if ( firstHandResult.highPair > secondHandResult.highPair )
			{
				return -1;
			}
			else if ( firstHandResult.highPair < secondHandResult.highPair )
			{
				return 1;
			}
			else if ( firstHandResult.lowPair > secondHandResult.lowPair )
			{
				return -1;
			}
			else if ( firstHandResult.lowPair < secondHandResult.lowPair )
			{
				return 1;
			}
			else
			{
				return 0;
			}
		}
		else if ( firstHandResult.handType == 1 )
		{
			if ( firstHandResult.pairVal > secondHandResult.pairVal )
			{
				return -1;
			}
			else if ( firstHandResult.pairVal < secondHandResult.pairVal )
			{
				return 1;
			}
			else
			{
				return 0;
			}
		}
		else if ( firstHandResult.handType == 0 )
		{
			if ( firstHandResult.handHigh > secondHandResult.handHigh )
			{
				return -1;
			}
			else if ( firstHandResult.handHigh < secondHandResult.handHigh )
			{
				return 1;
			}
			else
			{
				return 0;
			}
		}
	}

	return 0; // sort of a catch in case something breaks
}

function hasRoyalFlush ( cards, suits ) 
{
	var retObj = {
		handType: 9,
		card1: cards[0],
		card2: cards[1],
		card3: cards[2],
		card4: cards[3],
		card5: cards[4]
	};
	
	if ( suits[0] != 5 && suits[1] != 5 && suits[2] != 5 && suits[3] != 5)
	{
		return 0;
	}
	else
	{
		if ( cards[0].val != 1 )
		{
			return 0;
		}
		else if ( cards[1].val != 10 )
		{
			return 0;
		}
		else if ( cards[2].val != 11 )
		{
			return 0;
		}
		else if ( cards[3].val != 12 )
		{
			return 0;
		}
		else if ( cards[4].val != 13 )
		{
			return 0;
		}
		else
			return retObj;
	}
}

function hasStraightFlush ( cards, suits )
{
	var retObj = {
		handType: 8,
		highCard: 0,
		card1: cards[0],
		card2: cards[1],
		card3: cards[2],
		card4: cards[3],
		card5: cards[4]
	};
	
	if ( suits[0] != 5 && suits[1] != 5 && suits[2] != 5 && suits[3] != 5)
	{
		return 0;
	}
	else
	{
		var values = new Array(5);
		for ( var i = 0 ; i < cards.length ; i++ )
		{
			values[i] = cards[i].val - cards[0].val;
		}
	
		if ( values[1] != 1 )
		{
			return 0;
		}
		else if ( values[2] != 2 )
		{
			return 0;
		}
		else if ( values[3] != 3 )
		{
			return 0;
		}
		else if ( values[4] != 4 )
		{
			return 0;
		}
		else
		{
			retObj.highCard = cards[4].val;
			return retObj;
		}
		return 0;
	}
}

function hasFourOfAKind ( cards )
{
	var retObj = {
		handType: 7,
		matching: 0,
		kicker: 0,
		card1: cards[0],
		card2: cards[1],
		card3: cards[2],
		card4: cards[3],
		card5: cards[4]
	};
	
	for ( var i = 0; i < 2 ; i++ )
	{
		var isTheIndexCardDifferent = 0;
		var fourOfAKind = 0;
		for ( var j = i+1; j < 5 ;j++ )
		{
			if ( cards[i].val == cards[j].val )
			{
				fourOfAKind++;
				retObj.matching = cards[i].val;
				isTheIndexCardDifferent = 0;
			}
			else if ( isTheIndexCardDifferent && cards[i].val != cards[j].val )
			{
				retObj.kicker = cards[i].val;
				i++;
			}
			else
			{
				isTheIndexCardDifferent = j;
			}
		}
		if ( fourOfAKind >= 3 )
		{
			return retObj;
		}
	}
	return 0;
	
}

function hasFullHouse ( cards )
{
	var twoOfAKind = 0;
	var threeOfAKind = 0;
	
	for ( var i = 0; i < cards.length-1 ; i++ )
	{
		for ( var j = i+1; j < cards.length ; j++ )
		{
			if ( cards[i].val == cards[j].val )
			{
				if ( threeOfAKind == cards[i].val )
				{
					// Odds on this card pair has been checked already. Do nothing
				}
				else if ( twoOfAKind == 0 )
				{
					twoOfAKind = cards[i].val;
				}
				else if ( twoOfAKind == cards[i].val && threeOfAKind == 0 )
				{
					threeOfAKind = cards[i].val;
					twoOfAKind = 0;
				}
			}
		}
	}
	
	var retObj = {
		handType: 6,
		threeMatch: threeOfAKind,
		twoMatch: twoOfAKind,
		card1: cards[0],
		card2: cards[1],
		card3: cards[2],
		card4: cards[3],
		card5: cards[4]
	};
	
	if ( twoOfAKind != 0 && threeOfAKind != 0 )
	{
		alert ("Two of a kind = " + twoOfAKind + "\nThree of a kind = " + threeOfAKind);
		return retObj;
	}
	else
	{
		return 0;
	}
}

function hasFlush ( cards, suits ) 
{
	var retObj = {
		handType: 5,
		highCard: 0,
		card1: cards[0],
		card2: cards[1],
		card3: cards[2],
		card4: cards[3],
		card5: cards[4]
	};
	
	if ( cards[0].val == 1 )
	{
		retObj.highCard = 14;
	}
	else
	{
		retObj.highCard = cards[4].val;
	}
	
	if ( suits[0] != 5 && suits[1] != 5 && suits[2] != 5 && suits[3] != 5)
	{
		return 0;
	}
	else
	{
		return retObj;
	}
}

function hasStraight ( cards )
{
	var retObj = {
		handType: 4,
		highCard: 0,
		card1: cards[0],
		card2: cards[1],
		card3: cards[2],
		card4: cards[3],
		card5: cards[4]
	};
	
	if ( cards[0].val == 1 ) // Ace case. Might be royal straight
	{
		if ( cards[1].val == 10 && cards[2].val == 11 && cards[3].val == 12 && cards[4].val == 13 )
		{
			retObj.highCard = 14;
			return retObj;
		}
		else if ( cards[1].val == 2 && cards[2].val == 3 && cards[3].val == 4 && cards[4].val == 5 )
		{
			retObj.highCard = 5;
		}
		else 
		{
			return 0;
		}
	}
	else
	{
		var values = new Array(5);
		for ( var i = 0 ; i < cards.length ; i++ )
		{
			values[i] = cards[i].val - cards[0].val;
		}
	
		if ( values[1] != 1 )
		{
			return 0;
		}
		else if ( values[2] != 2 )
		{
			return 0;
		}
		else if ( values[3] != 3 )
		{
			return 0;
		}
		else if ( values[4] != 4 )
		{
			return 0;
		}
		else
		{
			retObj.highCard = cards[4].val;
			return retObj;
		}
	}
}

function hasThreeOfAKind ( cards )
{	
	var retObj = {
		handType: 3,
		threeOfAKindVal: 0,
		card1: cards[0],
		card2: cards[1],
		card3: cards[2],
		card4: cards[3],
		card5: cards[4]
	};
	
	for ( var i = 0; i < cards.length-1 ; i++ )
	{
		var matching = 0;
		for ( var j = i+1 ; j < cards.length ; j++ )
		{
			if ( cards[i].val == cards[j].val )
			{
				matching++;
				if ( matching >= 2 )
				{
					retObj.threeOfAKindVal = cards[i].val;
					return retObj;
				}
			}
		}
	}
	return 0;
}

function hasTwoPair ( cards )
{
	var pairs = 0;
	var retObj = {
		handType: 2,
		highPair: 0,
		lowPair: 0,
		card1: cards[0],
		card2: cards[1],
		card3: cards[2],
		card4: cards[3],
		card5: cards[4]
	};
	
	for ( var i = 0 ; i < cards.length-1 ; i++ )
	{
		for ( var j = i+1 ; j < cards.length ; j++ )
		{
			if ( cards[i].val == cards[j].val )
			{
				pairs++;
				if ( cards[i].val > retObj.lowPair )
				{
					retObj.highPair = cards[i].val;
				}
				else
				{
					retObj.highPair = retObj.lowPair;
					retObj.lowPair = cards[i].val;
				}
			}
		}
	}
	if ( pairs == 2 )
	{
		return retObj;
	}
	else
	{
		return 0;
	}
	
}

function hasPair ( cards )
{
	var retObj = {
		handType: 1,
		pairVal: 0,
		card1: cards[0],
		card2: cards[1],
		card3: cards[2],
		card4: cards[3],
		card5: cards[4]
	};
	
	for ( var i = 0 ; i < cards.length-1 ; i++ )
	{
		for ( var j = i+1 ; j < cards.length ; j++ )
		{
 			if ( cards[i].val == cards[j].val )
			{
				retObj.pairVal = cards[i].val;
				return retObj;
			}
		}
	}
	return 0;
}

function hasHighCard ( cards )
{
	if ( cards[0].val == 1 )
	{
		var retObj = {
			handType: 0,
			handHigh: 14,
		card1: cards[0],
		card2: cards[1],
		card3: cards[2],
		card4: cards[3],
		card5: cards[4]
		};
		return retObj;
	}
	else
	{
		var retObj = {
			handType: 0,
			handHigh: cards[4].val,
			card1: cards[0],
			card2: cards[1],
			card3: cards[2],
			card4: cards[3],
			card5: cards[4]
		};
		
		return retObj;
	}
}

// Returns a string that results from joining an array of object's attributes.
// PARAMETERS: a - the object to pass in
//             attr - the string of the attribute to join
function joinObj(a, attr) {
  var out = []; 
  for (var i=0; i<a.length; i++) { 
    out.push(a[i][attr]); 
  } 
  return out.join(", ");
}