function Game () {
	this.currentPot = 0;	// The pot of the hand. Resets every hand
	this.currentBet = 0;	// The current bet used for the raise function. Resets every betting stage.
	this.ante = 10;			// Ante feature. Currently not implemented.
	this.lastBetPlayer = 0;	// Used to determine the last player who raised. Needed to determine when to advance the betting stage.
	this.anteChip = 0;		// Who has the ante chip and bets first each betting stage. Currently not implemented.
	
	this.worstPossibleHand = 0; // NOT WORKING
	this.bestPossibleHand = null; // Set by flop time, turn time, river time
	
	this.currentState = 0;	// Which betting stage we are in. 0: pre-flop, 1: flop, 2: turn, 3: river, 4: evaluate winner
	
	this.river = new Array(5);	// Cards on the center of the table
	this.deck = new Array(52);	// The deck of cards to draw from
	var cardTypes = ['a', 'b', 'c', 'd'];	// Used for the images
	var cardSuits = ['c', 's', 'h', 'd']; 	// Used for the hand evaluators
	var counter = 0;
	// Number cards less than 10
	while ( counter < 32 )
	{
		this.deck[counter] = new Image();
		this.deck[counter].src = "img/cards/" + (Math.floor(counter / 4) + 2) + cardTypes[counter % 4] + ".png";	// The image of the card
		this.deck[counter].val = Math.floor(counter / 4 + 2);	// The number value of the card
		this.deck[counter].myType = cardSuits[counter % 4];		// The suit of the card
		counter++;
	}

	/* Face cards
	 * T = 10
	 * J = 11
	 * Q = 12
	 * K = 13
	 * A = 14 */
	this.deck[32] = new Image();
	this.deck[32].src = "img/cards/10a.png";
	this.deck[32].val = 'T';
	this.deck[32].myType = "c";
	
	this.deck[33] = new Image();
	this.deck[33].src = "img/cards/10b.png";
	this.deck[33].val = 'T';
	this.deck[33].myType = "s";
	
	this.deck[34] = new Image();
	this.deck[34].src = "img/cards/10c.png";
	this.deck[34].val = 'T';
	this.deck[34].myType = "h";
	
	this.deck[35] = new Image();
	this.deck[35].src = "img/cards/10d.png";
	this.deck[35].val = 'T';
	this.deck[35].myType = "d";
	
	this.deck[36] = new Image();
	this.deck[36].src = "img/cards/J1.png";
	this.deck[36].val = 'J';
	this.deck[36].myType = "c";

	this.deck[37] = new Image();
	this.deck[37].src = "img/cards/J2.png";
	this.deck[37].val = 'J';
	this.deck[37].myType = "s";

	this.deck[38] = new Image();
	this.deck[38].src = "img/cards/J3.png";
	this.deck[38].val = 'J';
	this.deck[38].myType = "h";

	this.deck[39] = new Image();
	this.deck[39].src = "img/cards/J4.png";
	this.deck[39].val = 'J';
	this.deck[39].myType = "d";

	this.deck[40] = new Image();
	this.deck[40].src = "img/cards/Q1.png";
	this.deck[40].val = 'Q';
	this.deck[40].myType = "c";

	this.deck[41] = new Image();
	this.deck[41].src = "img/cards/Q2.png";
	this.deck[41].val = 'Q';
	this.deck[41].myType = "s";

	this.deck[42] = new Image();
	this.deck[42].src = "img/cards/Q3.png";
	this.deck[42].val = 'Q';
	this.deck[42].myType = "h";

	this.deck[43] = new Image();
	this.deck[43].src = "img/cards/Q4.png";
	this.deck[43].val = 'Q';
	this.deck[43].myType = "d";

	this.deck[44] = new Image();
	this.deck[44].src = "img/cards/K1.png";
	this.deck[44].val = 'K';
	this.deck[44].myType = "c";

	this.deck[45] = new Image();
	this.deck[45].src = "img/cards/K2.png";
	this.deck[45].val = 'K';
	this.deck[45].myType = "s";

	this.deck[46] = new Image();
	this.deck[46].src = "img/cards/K3.png";
	this.deck[46].val = 'K';
	this.deck[46].myType = "h";

	this.deck[47] = new Image();
	this.deck[47].src = "img/cards/K4.png";
	this.deck[47].val = 'K';
	this.deck[47].myType = "d";

	this.deck[48] = new Image();
	this.deck[48].src = "img/cards/A1.png";
	this.deck[48].val = 'A';
	this.deck[48].myType = "c";

	this.deck[49] = new Image();
	this.deck[49].src = "img/cards/A2.png";
	this.deck[49].val = 'A';
	this.deck[49].myType = "s";

	this.deck[50] = new Image();
	this.deck[50].src = "img/cards/A3.png";
	this.deck[50].val = 'A';
	this.deck[50].myType = "h";

	this.deck[51] = new Image();
	this.deck[51].src = "img/cards/A4.png";
	this.deck[51].val = 'A';
	this.deck[51].myType = "d";
	
	// Raise the current game bet by adding to the current bet, 
	// and change whoever was the last player to bet so the other player has a chance to counter raise
	this.raiseBy = function (x) {
		this.currentBet += parseInt(x);
		if (this.lastBetPlayer == 0)
		{
			this.lastBetPlayer = 1;
		}
		else
		{
			this.lastBetPlayer = 0;
		}
		return this.currentBet;
	}

	// Raise the current game bet by replacing the current bet, 
	// and change whoever was the last player to bet so the other player has a chance to counter raise
	this.raiseTo = function (x) {
		if (this.lastBetPlayer == 0)
		{
			this.lastBetPlayer = 1;
		}
		else
		{
			this.lastBetPlayer = 0;
		}
		return this.currentBet = x;
	}

	// Pull a single card out of the deck
	this.getCard = function () {
		var card = this.deck[0];
		this.deck.splice(0,1);
		return card;
	}

	// Return an array of cards from the deck
	this.getCards = function (x) {
		var retArray = new Array(x);
		var counter = 0;
		while (counter < x)
		{
			retArray[counter] = this.getCard();
		} 
		return retArray;
	}
	 
	// Push a card back on to the back of the deck
	this.addCard = function(card) {
		var position = this.deck.length;
		this.deck[position] = card;
	}

	// Push an array of cards on to the back of the deck
	this.addCards = function(cards) {
		var counter = 0;
		while (counter < cards.length)
		{
			this.addCard(cards[counter]);
			counter++;
		}
	}
	 
	// Shuffle the cards by popping a card from a random position and assigning it to a temporary deck. 
	// Return the temporary deck, and save it as the game's deck.
	this.shuffle = function() {
	
		var tempDeck = new Array(this.deck.length);
		var counter = 0;
		var length = this.deck.length;
		var str = "";
		
		while (counter < length)
		{
			var position = Math.floor(Math.random()*1000) % this.deck.length;
			tempDeck[counter] = this.deck[position];
			this.deck.splice(position,1);
			counter++;
		}
		this.deck = tempDeck;
	
		return tempDeck;
	}
	
	// Return the cards from the river back to the deck, and reset the river
	this.resetRiver = function() {
		var counter = 0;
		while ( counter < this.river.length )
		{
			if (this.river[counter] == null)
			{
			}
			else
			{
				game.addCard(this.river[counter]);
			}
			counter++;
		}
		this.river = new Array(5);
	}
	
	// Reset the entire game, and change who has the ante chip.
	// TODO: After we implement the ante chip, we need to actually pull money from the players for the ante. Not implemented yet.
	this.reset = function () {
		this.resetRiver();
		if (this.anteChip == 0)
		{
			this.anteChip = 1;
		}
		else
		{
			this.anteChip = 0;
		}
		this.currentPot = 0;
		this.currentBet = 0;
		this.lastBetPlayer = this.anteChip;
		this.shuffle();
	}
	
}
